package com.doc.biz.controller;

import com.doc.biz.domain.*;
import com.doc.biz.service.*;
import com.doc.biz.vo.DocInfoVO;
import com.doc.biz.vo.DocumentVO;
import com.doc.chat.domain.ChatMsg;
import com.doc.common.annotation.Log;
import com.doc.common.config.EsConfig;
import com.doc.common.core.controller.BaseController;
import com.doc.common.core.domain.AjaxResult;
import com.doc.common.core.page.TableDataInfo;
import com.doc.common.enums.BusinessType;
import com.doc.common.utils.FileContentUtils;
import com.doc.common.utils.SecurityUtils;
import com.doc.common.utils.bean.BeanUtils;
import com.doc.common.utils.poi.ExcelUtil;
import com.doc.system.service.ISysConfigService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.elasticsearch.NoSuchIndexException;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.yaml.snakeyaml.util.UriEncoder;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.function.Function;

/**
 * 文件基本信息表Controller
 *
 * @author wukai
 * @date 2023-08-15
 */
@Api(tags = "文件基本信息表")
@RestController
@RequestMapping("/biz/info")
@Slf4j
public class DocInfoController extends BaseController {
    @Resource
    private IDocInfoService docInfoService;
    @Resource
    private IMongoService mongoService;
    @Resource
    private IEsDocInfoService esDocInfoService;
    @Resource
    private EsConfig esConfig;
    @Resource
    private IDocShareService shareService;
    @Resource
    private ISysConfigService configService;
    @Resource
    private IDocDirService dirService;

    /**
     * 文件上传
     *
     * @param file
     * @return
     */
    @ApiOperation("文件上传-单文件")
    @PostMapping("/upload")
    public AjaxResult uploadFile(@ApiParam(value = "文件", required = true) @RequestPart(value = "file") MultipartFile file, @ApiParam(value = "空间ID", required = true) @RequestParam Long spaceId, @ApiParam(value = "目录ID", required = true) @RequestParam Long dirId) {
        try {
            DocumentVO vo = mongoService.uploadFile(file);
            process(vo, spaceId, dirId);
            return success();
        } catch (Exception e) {
            log.error("文件上传失败：", e);
            return error(e.getMessage());
        }
    }

    /**
     * 多文件上传
     *
     * @param files
     * @return
     */
    @ApiOperation("文件上传-多文件")
    @PostMapping("/uploadFiles")
    public AjaxResult uploadFile(@ApiParam(value = "文件", required = true) @RequestPart(value = "files") List<MultipartFile> files, @ApiParam(value = "空间ID", required = true) @RequestParam Long spaceId, @ApiParam(value = "目录ID", required = true) @RequestParam Long dirId) {
        try {
            mongoService.uploadFiles(files).stream().forEach(vo -> {
                process(vo, spaceId, dirId);
            });
            return success();
        } catch (Exception e) {
            return error(e.getMessage());
        }
    }

    /**
     * 文件上传之后的操作
     *
     * @param vo      上传对象
     * @param spaceId 空间ID
     * @param dirId   目录ID
     */
    private void process(DocumentVO vo, Long spaceId, Long dirId) {
        DocInfo docInfo = new DocInfo();
        docInfo.setSpaceId(spaceId);
        docInfo.setDirId(dirId);
        docInfo.setFileId(vo.getFileId());
        docInfo.setFileName(vo.getFileName());
        docInfo.setFileSize(vo.getFileSize());
        docInfo.setFileType(vo.getSuffix());
        docInfo.setCreateBy(SecurityUtils.getUsername());
        docInfoService.insertDocInfo(docInfo);

        DocumentVO documentVO = mongoService.downloadFile(vo.getFileId());
        insertEs(documentVO.getData(), docInfo.getFileType(), docInfo.getDocId(), spaceId);
    }

    /**
     * 文件下载
     *
     * @param fileId fileId
     * @return
     */
    @ApiOperation("文件下载")
    @GetMapping("/download/{fileId}")
    public ResponseEntity<Object> download(@PathVariable(name = "fileId") String fileId) {
        return down(fileId, true);
    }

    private ResponseEntity<Object> down(String fileId, boolean down) {
        DocumentVO vo = mongoService.downloadFile(fileId);
        if (Objects.nonNull(vo)) {
            String disposition = down ? "attachment;" : "" + "filename=\"" + UriEncoder.encode(vo.getFileName()) + "\"";
            return ResponseEntity.ok().header(HttpHeaders.CONTENT_DISPOSITION, disposition).header(HttpHeaders.CONTENT_TYPE, vo.getContentType()).header(HttpHeaders.CONTENT_LENGTH, vo.getFileSize() + "").header("Connection", "close").body(vo.getData());
        } else {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body("file does not exist");
        }
    }

    /**
     * 文件下载
     *
     * @param fileId fileId
     * @return
     */
    @ApiOperation("文件预览")
    @GetMapping("/preview/{fileId}")
    public ResponseEntity<Object> preview(@PathVariable(name = "fileId") String fileId) {
        return down(fileId, false);
    }

    /**
     * 文件移动
     */
    @ApiOperation("文件移动")
    @GetMapping("/move")
    @ApiImplicitParams({@ApiImplicitParam(name = "docId", value = "文件ID", required = true), @ApiImplicitParam(name = "spaceId", value = "空间ID"), @ApiImplicitParam(name = "dirId", value = "新目录ID", required = true)})
    public AjaxResult move(Long docId, Long dirId) {
        DocInfo info = docInfoService.selectDocInfoByDocId(docId);
        info.setDirId(dirId);

        info.setUpdateBy(SecurityUtils.getUsername());
        docInfoService.updateDocInfo(info);
        return success();
    }

    /**
     * 复制文件
     */
    @ApiOperation("文件复制")
    @GetMapping("/copy")
    @ApiImplicitParams({@ApiImplicitParam(name = "docId", value = "文件ID", required = true), @ApiImplicitParam(name = "spaceId", value = "空间ID"), @ApiImplicitParam(name = "dirId", value = "新目录ID", required = true)})
    public AjaxResult copy(Long docId, Long spaceId, Long dirId) {
        DocInfo info = docInfoService.selectDocInfoByDocId(docId);
        info.setDocId(null);
        info.setSpaceId(spaceId);
        info.setDirId(dirId);
        docInfoService.insertDocInfo(info);

        return success();
    }

    /**
     * 文件分享
     */
    @ApiOperation("文件分享获取详情")
    @GetMapping("/share/{docId}")
    public List<DocShare> shareInfo(@ApiParam(value = "文件ID", required = true) @PathVariable(name = "docId") Long docId) {
        return shareService.selectDocShareListByDocId(docId);
    }

    /**
     * 文件分享
     */
    @ApiOperation("文件分享添加人员")
    @PostMapping("/share/{docId}")
    public AjaxResult share(@ApiParam(value = "文件ID", required = true) @PathVariable(name = "docId") Long docId, @ApiParam(value = "分享人员", required = true) @RequestBody List<DocShare> shares) {
        shareService.deleteDocShareByDocId(docId);

        String content = configService.selectConfigByKey("msg.share.content");
        shares.forEach(s -> {
            s.setDocId(docId);
            ChatMsg msg = new ChatMsg();
            msg.setMsgType("1");
            msg.setToId(s.getUserId());
            msg.setContent(docId + "");
            msg.setFromId(SecurityUtils.getUserId());
            shareService.insertDocShare(s);
        });

        return success();
    }

    /**
     * 复制文件
     */
    @ApiOperation("文件重命名")
    @GetMapping("/rename")
    @ApiImplicitParams({@ApiImplicitParam(name = "docId", value = "文件ID", required = true), @ApiImplicitParam(name = "name", value = "新文件名", required = true)})
    public AjaxResult rename(Long docId, String name) {
        DocInfo info = new DocInfo();
        info.setDocId(docId);
        info.setFileName(name);

        info.setUpdateBy(SecurityUtils.getUsername());

        docInfoService.insertDocInfo(info);

        return success();
    }

    /**
     * 查询文件基本信息表列表
     */
//    @ApiOperation("文件列表")
    //@PreAuthorize("@ss.hasPermi('biz:info:list')")
    @GetMapping("/list")
    public TableDataInfo list(DocInfo docInfo) {
        startPage();
        List<DocInfo> list = docInfoService.selectDocInfoList(docInfo);
        return getDataTable(list);
    }

    /**
     * 文件搜索
     */
    @ApiOperation("文件搜索")
    //@PreAuthorize("@ss.hasPermi('biz:info:list')")
    @GetMapping("/search")
    public TableDataInfo search(@ApiParam(value = "搜索关键字", required = true) @RequestParam String keyword) {
        Map<String, Object> temp = dirService.selectDirByUser(SecurityUtils.getUserId());
        List<DocDir> dirList = (List<DocDir>) temp.get("dir");
        List<DocSpace> spaceList = (List<DocSpace>) temp.get("space");
        Map<Long, DocDir> dirMap = new HashMap<>(16);
        Map<Long, DocSpace> spaceMap = new HashMap<>(16);
        List<Long> dirIds = new ArrayList<>();
        dirList.forEach(dir -> {
            dirIds.add(dir.getDirId());
            dirMap.put(dir.getDirId(), dir);
        });

        spaceList.forEach(space -> {
            spaceMap.put(space.getSpaceId(), space);
        });

        Map<String, Object> map = new HashMap<>(4);
        map.put("dirIds", dirIds);

        DocInfo docInfo = new DocInfo();
        docInfo.setFileName(keyword);
        docInfo.setParams(map);
        startPage();
        List<DocInfo> list = docInfoService.selectDocInfoList(docInfo);
        List<DocInfoVO> result = new ArrayList<>();
        list.forEach(info -> {
            DocInfoVO vo = new DocInfoVO();
            BeanUtils.copyProperties(info, vo);

            vo.setDir(dirMap.get(vo.getDirId()));
            vo.setSpace(spaceMap.get(vo.getSpaceId()));

            result.add(vo);
        });
        return getDataTable(result);
    }

    /**
     * 查询文件基本信息表列表
     */
    @ApiOperation("根据目录ID查询文件列表")
    //@PreAuthorize("@ss.hasPermi('biz:info:list')")
    @GetMapping("/list/{dirId}")
    public TableDataInfo list4dir(@PathVariable("dirId") Long dirId) {
        startPage();
        DocInfo docInfo = new DocInfo();
        docInfo.setDirId(dirId);
        List<DocInfo> list = docInfoService.selectDocInfoList(docInfo);
        return getDataTable(list);
    }

    /**
     * 导出文件基本信息表列表
     */
//    @ApiOperation("导出文件基本信息表列表")
    //@PreAuthorize("@ss.hasPermi('biz:info:export')")
    @Log(title = "文件基本信息表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, DocInfo docInfo) {
        List<DocInfo> list = docInfoService.selectDocInfoList(docInfo);
        ExcelUtil<DocInfo> util = new ExcelUtil<DocInfo>(DocInfo.class);
        util.exportExcel(response, list, "文件基本信息表数据");
    }

    /**
     * 获取文件基本信息表详细信息
     */
    @ApiOperation("文件详细信息")
    //@PreAuthorize("@ss.hasPermi('biz:info:query')")
    @GetMapping(value = "/{docId}")
    public AjaxResult getInfo(@PathVariable("docId") Long docId) {
        return success(docInfoService.selectDocInfoByDocId(docId));
    }

    /**
     * 新增文件基本信息表
     */
//    @ApiOperation("新增文件基本信息表")
    //@PreAuthorize("@ss.hasPermi('biz:info:add')")
    @Log(title = "文件基本信息表", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody DocInfo docInfo) {
        return toAjax(docInfoService.insertDocInfo(docInfo));
    }

    /**
     * 修改文件基本信息表
     */
//    @ApiOperation("修改文件基本信息表")
    //@PreAuthorize("@ss.hasPermi('biz:info:edit')")
    @Log(title = "文件基本信息表", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody DocInfo docInfo) {
        //TODO 文件内容修改的ES处理
        docInfo.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(docInfoService.updateDocInfo(docInfo));
    }

    /**
     * 删除文件基本信息表
     */
    @ApiOperation("文件删除")
    //@PreAuthorize("@ss.hasPermi('biz:info:remove')")
    @Log(title = "文件基本信息表", businessType = BusinessType.DELETE)
    @DeleteMapping("/{docIds}")
    public AjaxResult remove(@PathVariable Long[] docIds) {
        for (Long docId : docIds) {
            DocInfo info = docInfoService.selectDocInfoByDocId(docId);
            //删除mongo记录
            mongoService.removeFile(info.getFileId());

            try {
                //删除ES记录
                String indexName = "docs_" + info.getSpaceId();
                esConfig.setIndexName(indexName);
                esDocInfoService.deleteById(docId);
            } catch (NoSuchIndexException e) {
                //不用管，表示没这个索引
            }
        }
        return toAjax(docInfoService.deleteDocInfoByDocIds(docIds));
    }


    /**
     * 文件内容入es库
     *
     * @param data    二进制文件内容
     * @param ext     扩展名
     * @param docId   文档ID
     * @param spaceId 空间ID
     */
    private void insertEs(byte[] data, String ext, Long docId, Long spaceId) {
        //组装ES索引名
        String indexName = "docs_" + spaceId;

        Map<String, Function<byte[], String>> handlerMap = new HashMap<>(16);
        handlerMap.put(".docx", FileContentUtils::getContentDocx);
        handlerMap.put(".doc", FileContentUtils::getContentDoc);
        handlerMap.put(".wps", FileContentUtils::getContentWps);
        handlerMap.put(".txt", FileContentUtils::getContentTxt);

        Function<byte[], String> handler = handlerMap.get(ext);
        if (handler != null) {
            String content = handler.apply(data);
            System.err.println(content);
            EsDocInfo esDocInfo = new EsDocInfo(docId, content);
            esConfig.setIndexName(indexName);
            esDocInfoService.save(esDocInfo);
        }
    }
}
